#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os
import errno
import fcntl
import socket

from checkbox.tests.cases import TestCase
from checkbox.tests.process import ProcessMixin

from checkbox.connector.util import (
    do_accept,
    do_connect,
    )


class TestDoConnect(TestCase):

    def getInbound(self, host="127.0.0.1", port=0):
        inbound = socket.socket()
        inbound.bind((host, port))
        inbound.listen(0)
        return inbound

    def test_error(self):
        sock = socket.socket()
        address = ("127.0.0.1", 0)
        try:
            do_connect(sock, address)
        except socket.error, error:
            self.assertEquals(error.errno, errno.ECONNREFUSED)
        else:
            self.fail("socket.error not raised")

    def test_blocking(self):
        inbound = self.getInbound()
        address = inbound.getsockname()

        outbound = socket.socket()
        do_connect(outbound, address)

    def test_nonblocking(self):
        inbound = self.getInbound()
        address = inbound.getsockname()

        outbound = socket.socket()
        flags = fcntl.fcntl(outbound.fileno(), fcntl.F_GETFL)
        fcntl.fcntl(outbound.fileno(), fcntl.F_SETFL, flags | os.O_NONBLOCK)
        try:
            do_connect(outbound, address)
        except socket.error, error:
            self.assertEquals(error.errno, errno.EINPROGRESS)
        else:
            self.fail("socket.error not raised")


class TestDoAccept(ProcessMixin, TestCase):

    def getInbound(self, host="127.0.0.1", port=0):
        inbound = socket.socket()
        inbound.bind((host, port))
        inbound.listen(0)
        return inbound

    def test_none(self):
        sock = socket.socket()
        result = do_accept(sock)
        self.assertEquals(result, (None, (None, 0)))
        sock.close()

    def test_error(self):
        inbound = self.getInbound()
        try:
            do_accept(inbound, 0)
        except socket.error, error:
            self.assertEquals(error.errno, errno.ETIMEDOUT)
            inbound.close()
        else:
            self.fail("socket.error not raised")

    def test_blocking(self):
        inbound = self.getInbound()
        address = inbound.getsockname()

        if not self.forkChild():
            outbound = socket.socket()
            outbound.connect(address)
            outbound.close()
            os._exit(0)

        sock, address = do_accept(inbound, 1)
        self.assertTrue(isinstance(sock, socket.socket))
        self.assertEquals(address[0], "127.0.0.1")

    def test_nonblocking(self):
        inbound = self.getInbound()
        flags = fcntl.fcntl(inbound.fileno(), fcntl.F_GETFL)
        fcntl.fcntl(inbound.fileno(), fcntl.F_SETFL, flags | os.O_NONBLOCK)
        try:
            do_accept(inbound, 0)
        except socket.error, error:
            self.assertEquals(error.errno, errno.EAGAIN)
            inbound.close()
        else:
            self.fail("socket.error not raised")
