#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "CommandHandler",
    "HandlerType",
    "PipeHandler",
    "ReapHandler",
    "SignalHandler",
    "SockHandler",
    ]

from checkbox.lib.enum import Enum


HandlerType = Enum(
    ("READ", 1),
    "WRITE",
    "READ_WRITE",
    )


class Handler:

    __slots__ = (
        "function",
        )

    def __init__(self):
        self.function = None


class CommandHandler(Handler):

    __slots__ = (
        "number",
        )

    def __init__(self):
        super(CommandHandler, self).__init__()
        self.number = 0


class PipeHandler(Handler):

    __slots__ = (
        "pipe_end",
        "type",
        "pid_entry",
        "call_handler",
        "in_handler",
        )

    def __init__(self):
        super(PipeHandler, self).__init__()
        self.pipe_end = -1
        self.type = 0
        self.pid_entry = None
        self.call_handler = False
        self.in_handler = False


class ReapHandler(Handler):

    __slots__ = (
        "number",
        )

    def __init__(self):
        super(ReapHandler, self).__init__()
        self.number = 0


class SignalHandler(Handler):

    __slots__ = (
        "is_blocked",
        "is_pending",
        "number",
        )

    def __init__(self):
        super(SignalHandler, self).__init__()
        self.number = 0
        self.is_blocked = False
        self.is_pending = False


class SockHandler(Handler):

    __slots__ = (
        "call_handler",
        "is_pending",
        "sock",
        )

    def __init__(self):
        super(SockHandler, self).__init__()
        self.sock = None
        self.is_pending = False
        self.call_handler = False
