#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "MessageCallback",
    ]


class MessageCallback:

    __slots__ = (
        "_data",
        "_function",
        "_message",
        "_service",
        )

    def __init__(self, function, data=None):
        self._function = function
        self._data = data

    def get_data(self):
        """Get the callback data."""
        return self._data

    def set_data(self, data):
        """Set the callback data.

        :param data: Data to set.
        """
        self._data = data

    def get_message(self):
        """Get the callback message."""
        self._message.get()

    def set_message(self, message):
        """Set the callback message.

        :param message: Message to set.
        """
        self._message = message

    def do_callback(self):
        """Do the callback function, passing self as argument."""
        self._function(self)

    def cancel_message(self):
        """Cancel the callback message."""
        if self._message.get():
            self._message.cancel_message()
