#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.daemon.type import DaemonType

from checkbox.daemon.client import Client, ClientInfo


class ClientDaemonInfo(ClientInfo):

    type = DaemonType.MASTER

    def _do_locate(self):
        return self._get_daemon_info()


class ClientConnectorInfo(ClientInfo):

    type = DaemonType.COLLECTOR

    def _do_locate(self):
        return self._get_connector_info()


class TestClientInfo(TestCase):

    def test_locate_empty(self):
        info = ClientDaemonInfo()
        self.assertFalse(info.locate())
        self.assertTrue(info.is_local())
        self.assertNotEquals(info.get_name(), None)
        self.assertNotEquals(info.get_fullhostname(), None)

    def test_locate_daemon_name(self):
        info = ClientDaemonInfo("subsystem@foo.bar.com")
        self.assertFalse(info.locate())
        self.assertFalse(info.is_local())
        self.assertEquals(info.get_name(), "subsystem@foo.bar.com")
        self.assertEquals(info.get_fullhostname(), "foo.bar.com")

    def test_locate_hostname(self):
        info = ClientDaemonInfo("foo")
        self.assertFalse(info.locate())
        self.assertFalse(info.is_local())
        self.assertEquals(info.get_name(), "foo")
        #self.assertEquals(info.get_fullhostname(), "foo")

    def test_locate_full_hostname(self):
        info = ClientDaemonInfo("foo.bar.com")
        self.assertFalse(info.locate())
        self.assertFalse(info.is_local())
        self.assertEquals(info.get_name(), "foo.bar.com")
        #self.assertEquals(info.get_fullhostname(), "foo.bar.com")

    def dump_info(self, info):
        print 'name: %s' % info.get_name()
        print 'address: %s' % info.get_address()
        print 'hostname: %s' % info.get_hostname()
        print 'fullhostname: %s' % info.get_fullhostname()
        print 'pool: %s' % info.get_pool()
        print 'is_local: %s' % info.is_local()
