#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.daemon.messagecallback import MessageCallback


class TestMessageCallback(TestCase):

    def setUp(self):
        self.data = None
        self.count = 0

    def function(self, message):
        self.data = message.get_data()
        self.count += 1

    def test_function(self):
        message = MessageCallback(self.function)
        self.assertEquals(self.data, None)
        self.assertEquals(self.count, 0)

        message.do_callback()
        self.assertEquals(self.data, None)
        self.assertEquals(self.count, 1)

    def test_function_with_data(self):
        message = MessageCallback(self.function, "foo")
        self.assertEquals(self.data, None)
        self.assertEquals(self.count, 0)

        message.do_callback()
        self.assertEquals(self.data, "foo")
        self.assertEquals(self.count, 1)

    def test_do_callback(self):
        message = MessageCallback(self.function)
        self.assertEquals(self.count, 0)

        message.do_callback()
        self.assertEquals(self.count, 1)

        message.do_callback()
        self.assertEquals(self.count, 2)

    def test_data(self):
        message = MessageCallback(self.function)
        self.assertEquals(message.get_data(), None)

        message.set_data("foo")
        self.assertEquals(message.get_data(), "foo")
