#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.daemon.waitpid import WaitPid, WaitPidQueue


class TestWaitPid(TestCase):

    def test_init(self):
        wait_pid = WaitPid(1)
        self.assertEquals(wait_pid.child_pid, 1)
        self.assertEquals(wait_pid.exit_status, 0)

        wait_pid = WaitPid(2, 1)
        self.assertEquals(wait_pid.child_pid, 2)
        self.assertEquals(wait_pid.exit_status, 1)

    def test_cmp(self):
        wait_pid1 = WaitPid(1)
        wait_pid2 = WaitPid(1)
        self.assertEquals(wait_pid1, wait_pid2)

        wait_pid2 = WaitPid(2)
        self.assertNotEquals(wait_pid1, wait_pid2)


class TestWaitPidQueue(TestCase):

    def test_init(self):
        queue = WaitPidQueue()

    def test_enqueue(self):
        wait_pid = WaitPid(1)

        queue = WaitPidQueue()
        queue.enqueue(wait_pid)
        self.assertTrue(wait_pid in queue)

    def test_enqueue(self):
        wait_pid = WaitPid(1)

        queue = WaitPidQueue()
        queue.enqueue(wait_pid)
        self.assertTrue(wait_pid in queue)

        dequeued_wait_pid = queue.dequeue()
        self.assertEquals(dequeued_wait_pid, wait_pid)
        self.assertFalse(wait_pid in queue)
