#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "Buffer",
    ]

from checkbox.io.stream import Stream


class Buffer(Stream):

    __slots__ = (
        "_buffer",
        "_buffers",
        )

    def __init__(self, buffer=""):
        super(Buffer, self).__init__()
        self._buffer = buffer
        self._buffers = []

    def getBytes(self, size=-1):
        """See Stream.getBytes"""
        if self._buffers:
            self._buffer += "".join(self._buffers)

        if size < 0:
            index = len(self._buffer)

        else:
            index = min(size, len(self._buffer))

        bytes = self._buffer[:index]
        self._buffers = [self._buffer[index:]]
        self._buffer = ""

        return bytes

    def getEnd(self):
        """See Stream.getEnd"""
        return True

    def peek(self):
        """See Stream.peek"""
        if self._buffers:
            self._buffer += "".join(self._buffers)
            self._buffers = []

        if not len(self._buffer):
            return None

        return self._buffer[0]

    def search(self, sub):
        """See Stream.search"""
        if self._buffers:
            self._buffer += "".join(self._buffers)
            self._buffers = []

        index = self._buffer.find(sub)
        if index < 0:
            return None

        bytes = self._buffer[:index]
        self._buffer = self._buffer[index + len(sub):]

        return bytes

    def putBytes(self, bytes):
        """See Stream.putBytes"""
        if not bytes:
            return 0

        self._buffers.append(bytes)
        return len(bytes)

    def putEnd(self):
        """See Stream.putEnd"""
        return True
