#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "get_port_range",
    ]

import logging

from checkbox.lib import param


def get_port_range(outbound):
    """Get the low and high port range for inbound or outbound connections.

    :param outbound: Whether this is for an outbound connection.
    """
    param_prefix = "OUTPUT" if outbound else "INPUT"
    low_param = "%s_LOW_PORT" % param_prefix
    high_param = "%s_HIGH_PORT" % param_prefix
    low_port = param.getInteger(low_param, 0)
    high_port = param.getInteger(high_param, 0)

    # Fallback to the default IO_LOW_PORT and IO_HIGH_PORT
    if low_port == 0 and high_port == 0:
        low_param = "IO_LOW_PORT"
        high_param = "IO_HIGH_PORT"
        low_port = param.getInteger(low_param, 0)
        high_port = param.getInteger(high_param, 0)

    logging.debug(
        "Using %s (%d) and %s (%d)",
        low_param, low_port, high_param, high_port)

    # Check bounds
    if low_port < 0 or high_port < 0 or low_port > high_port:
        logging.info(
            "Invalid port range (%d, %d)", low_port, high_port)
        return (0, 0)

    # Check privileges
    if low_port < 1024 and high_port >= 1024:
        logging.info(
            "Mixed privileges in port range (%d, %d)", low_port, high_port)

    return (low_port, high_port)
