#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "SockMixin",
    ]

import os

from checkbox.io.tests.stream import StreamMixin


class SockMixin(StreamMixin):

    def makeSock(self):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def run_in_child(self, function, *args, **kwargs):
        pid = os.fork()
        if pid == 0:
            function(*args, **kwargs)
            os._exit(0)

    def test_get_host_and_port(self):
        # Empty socket
        sock = self.makeSock()
        host, port = sock.get_host_and_port()
        self.assertEquals(host, None)
        self.assertEquals(port, 0)
        host, port = sock.get_peer_host_and_port()
        self.assertEquals(host, None)
        self.assertEquals(port, 0)

        # Outbound socket
        sock = self.makeSock()
        self.assertTrue(sock.connect("127.0.0.1", 0))
        host, port = sock.get_host_and_port()
        self.assertNotEquals(host, None)
        self.assertNotEquals(port, 0)
        host, port = sock.get_peer_host_and_port()
        self.assertEquals(host, "127.0.0.1")
        self.assertEquals(port, 0)

    def test_connect(self):
        stream = self.makeSock()
        self.assertFalse(stream.connect(None, -1, True))
        self.assertFalse(stream.connect(None, 0, True))
        self.assertFalse(stream.connect("", 0, True))
        self.assertFalse(stream.connect("127.0.0.1", -1, True))
        self.assertFalse(stream.connect("127.0.0.1", -1, False))
