#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "StreamMixin",
    ]


class StreamMixin:

    def getReader(self):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def getWriter(self):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def test_bytes(self):
        writer = self.getWriter()
        self.assertEquals(writer.putBytes("foo"), 3)
        self.assertEquals(writer.putBytes("bar"), 3)
        self.assertTrue(writer.putEnd())

        reader = self.getReader()
        self.assertEquals(reader.getBytes(3), "foo")
        self.assertEquals(reader.getBytes(3), "bar")
        self.assertEquals(reader.getBytes(3), "")
        self.assertTrue(reader.getEnd())

    def test_address(self):
        writer = self.getWriter()
        self.assertTrue(writer.putAddress("127.0.0.1"))
        self.assertTrue(writer.putEnd())

        reader = self.getReader()
        self.assertEquals(reader.getAddress(), "127.0.0.1")
        self.assertEquals(reader.getAddress(), None)
        self.assertTrue(reader.getEnd())

    def test_boolean(self):
        writer = self.getWriter()
        self.assertTrue(writer.putBoolean(True))
        self.assertTrue(writer.putBoolean(False))
        self.assertTrue(writer.putEnd())

        reader = self.getReader()
        self.assertEquals(reader.getBoolean(), True)
        self.assertEquals(reader.getBoolean(), False)
        self.assertEquals(reader.getBoolean(), None)
        self.assertTrue(reader.getEnd())

    def test_integer(self):
        writer = self.getWriter()
        self.assertTrue(writer.putInteger(1))
        self.assertTrue(writer.putInteger(-1))
        self.assertTrue(writer.putEnd())

        reader = self.getReader()
        self.assertEquals(reader.getInteger(), 1)
        self.assertEquals(reader.getInteger(), -1)
        self.assertEquals(reader.getInteger(), None)
        self.assertTrue(reader.getEnd())

    def test_real(self):
        writer = self.getWriter()
        self.assertTrue(writer.putReal(1.0))
        self.assertTrue(writer.putReal(-1.1))
        self.assertTrue(writer.putEnd())

        reader = self.getReader()
        self.assertEquals(reader.getReal(), 1.0)
        self.assertEquals(reader.getReal(), -1.1)
        self.assertEquals(reader.getReal(), None)
        self.assertTrue(reader.getEnd())

    def test_string(self):
        writer = self.getWriter()
        self.assertTrue(writer.putString("foo"))
        self.assertTrue(writer.putString(None))
        self.assertTrue(writer.putString("bar"))
        self.assertTrue(writer.putEnd())

        reader = self.getReader()
        self.assertEquals(reader.getString(), "foo")
        self.assertEquals(reader.getString(), None)
        self.assertEquals(reader.getString(), "bar")
        self.assertEquals(reader.getString(), None)
        self.assertTrue(reader.getEnd())
