#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os

from checkbox.tests.cases import TestCase
from checkbox.tests.env import EnvMixin

from checkbox.io.port import get_port_range


class TestPort(EnvMixin, TestCase):

    def setUp(self):
        super(TestPort, self).setUp()

        for name in ("BIND_ALL_INTERFACES",
                "OUTPUT_LOW_PORT", "OUTPUT_HIGH_PORT",
                "INPUT_LOW_PORT", "INPUT_HIGH_PORT"):
            if name in os.environ:
                del os.environ[name]

    def test_get_port_range_outbound(self):
        low_port, high_port = get_port_range(True)
        self.assertEquals(low_port, 0)
        self.assertEquals(high_port, 0)

    def test_get_port_range_inbound(self):
        low_port, high_port = get_port_range(False)
        self.assertEquals(low_port, 0)
        self.assertEquals(high_port, 0)

    def test_get_port_range_low_port(self):
        # Low port, no high port
        os.environ["OUTPUT_LOW_PORT"] = "1024"
        low_port, high_port = get_port_range(True)
        self.assertEquals(low_port, 0)
        self.assertEquals(high_port, 0)

    def test_get_port_range_low_gt_high_port(self):
        # Low port > high port
        os.environ["OUTPUT_LOW_PORT"] = "1024"
        os.environ["OUTPUT_HIGH_PORT"] = "1023"
        low_port, high_port = get_port_range(True)
        self.assertEquals(low_port, 0)
        self.assertEquals(high_port, 0)

    def test_get_port_range_low_eq_high_port(self):
        # Low port == high port
        os.environ["OUTPUT_LOW_PORT"] = "1024"
        os.environ["OUTPUT_HIGH_PORT"] = "1024"
        low_port, high_port = get_port_range(True)
        self.assertEquals(low_port, 1024)
        self.assertEquals(high_port, 1024)

    def test_get_port_range_low_lt_high_port(self):
        # Low port < high port
        os.environ["OUTPUT_LOW_PORT"] = "1024"
        os.environ["OUTPUT_HIGH_PORT"] = "1025"
        low_port, high_port = get_port_range(True)
        self.assertEquals(low_port, 1024)
        self.assertEquals(high_port, 1025)
