#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os

from checkbox.tests.cases import TestCase
from checkbox.tests.env import EnvMixin

from checkbox.io.sock import SockAddress
from checkbox.io.socktcp import SockTCP

from checkbox.io.tests.sock import SockMixin


class TestSockTCP(EnvMixin, SockMixin, TestCase):

    def setUp(self):
        super(TestSockTCP, self).setUp()

        os.environ["OUTPUT_LOW_PORT"] = "2000"
        os.environ["OUTPUT_HIGH_PORT"] = "2999"
        os.environ["INPUT_LOW_PORT"] = "3000"
        os.environ["INPUT_HIGH_PORT"] = "3999"

        self.reader = SockTCP()
        self.writer = SockTCP()
        self.writer.connectSock(self.reader)

    def getReader(self):
        return self.reader

    def getWriter(self):
        return self.writer

    def makeSock(self):
        return SockTCP()

    def test_get_host_and_port(self):
        super(TestSockTCP, self).test_get_host_and_port()

        # Inbound socket
        def connect(host, port):
            sock = self.makeSock()
            sock.connect(host, port)

        sock = self.makeSock()
        self.assertTrue(sock.bind(SockAddress.LOOPBACK))
        self.assertTrue(sock.listen())
        host, port = sock.get_host_and_port()

        self.run_in_child(connect, host, port)
        peer = sock.accept()
        peer_host, peer_port = peer.get_host_and_port()
        self.assertEquals(host, peer_host)
        self.assertEquals(port, peer_port)

    def test_nobuffer(self):
        # Send size
        writer = self.getWriter()
        self.assertTrue(writer.putBytesUnbuffered("abc"))
        self.assertTrue(writer.putBytesUnbuffered("def"))

        reader = self.getReader()
        self.assertEquals(reader.getBytesUnbuffered(6), "abc")
        self.assertEquals(reader.getBytesUnbuffered(6), "def")

        # Don't send size
        self.assertTrue(writer.putBytesUnbuffered("abc", False))
        self.assertTrue(writer.putBytesUnbuffered("def", False))
        self.assertEquals(reader.getBytesUnbuffered(6, False), "abcdef")

    def test_serialize(self):
        writer = self.getWriter()
        self.assertTrue(writer.putBytesUnbuffered("abc"))

        # Serialize reader and unserialize new reader
        reader = self.getReader()
        string = reader.serialize()
        new_reader = self.makeSock()
        new_reader.unserialize(string)

        self.assertEquals(new_reader.getBytesUnbuffered(6), "abc")
