#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from checkbox.tests.cases import TestCase

from checkbox.io.sock import SockAddress
from checkbox.io.sockudp import SockUDP

from checkbox.io.tests.sock import SockMixin


class TestSockUDP(SockMixin, TestCase):

    def setUp(self):
        self.reader = SockUDP()
        self.reader.bind(SockAddress.LOOPBACK, 2000)

        self.writer = SockUDP()
        self.writer.connect(SockAddress.LOOPBACK, 2000)

    def tearDown(self):
        self.writer.close()
        self.reader.close()

    def getReader(self):
        return self.reader

    def getWriter(self):
        return self.writer

    def makeSock(self):
        return SockUDP()

    def test_getBytes(self):
        def do_write():
            writer = self.getWriter()
            writer.putBytes("abc")
            writer.putEnd()

        self.run_in_child(do_write)

        reader = self.getReader()
        self.assertEquals(reader.getBytes(3), "abc")
        self.assertTrue(reader.getEnd())

    def test_putBytes(self):
        def do_read():
            reader = self.getReader()
            reader.getBytes(3)
            reader.getEnd()

        self.run_in_child(do_read)

        writer = self.getWriter()
        writer.putBytes("abc")
        writer.putEnd()

    def test_serialize(self):
        def do_write():
            writer = self.getWriter()
            writer.putBytes("abc")
            writer.putEnd()

        self.run_in_child(do_write)

        # Serialize reader and unserialize new reader
        reader = self.getReader()
        string = reader.serialize()
        new_reader = self.makeSock()
        new_reader.unserialize(string)

        self.assertEquals(reader.getBytes(3), "abc")
        self.assertTrue(reader.getEnd())
