#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "FileEntry",
    "FileState",
    "FileStateInfo",
    ]

from checkbox.lib.enum import Enum


FILE_STATE_SIGNATURE = "FileState"
FILE_STATE_VERSION = 1


FileStatus = Enum(
    ("ERROR", -1),
    "NO_CHANGE",
    "GROWN",
    "SHRUNK",
    )


class FileEntry:

    __slots__ = (
        "filename",
        "is_initialized",
        "is_valid",
        "have_read_event",
        "journal_size",
        )

    def __init__(self, filename):
        self.filename = filename
        self.is_initialized = False
        self.is_valid = False
        self.have_read_event = False
        self.journal_size = 0


class FileState:

    __slots__ = (
        "info",
        )

    def __init__(self, info=None):
        self.info = info

    def __getattr__(self, name):
        return getattr(self.info, name)

    def __setattr__(self, name, value):
        if name in FileState.__slots__:
            return super(FileState, self).__setattr__(name, value)

        return setattr(self.info, name, value)


class FileStateInfo:

    __slots__ = (
        "signature",
        "version",
        "base_path",
        "unique_id",
        "sequence",
        "rotation",
        "max_rotations",
        "inode",
        "ctime",
        "size",
        "offset",
        "position",
        "record",
        "update_time",
        )

    def __init__(self):
        self.signature = FILE_STATE_SIGNATURE
        self.version = FILE_STATE_VERSION
        self.base_path = ""
        self.unique_id = ""
        self.sequence = 0
        self.rotation = 0
        self.max_rotations = 0
        self.inode = 0
        self.ctime = 0
        self.size = 0
        self.offset = 0
        self.position = 0
        self.record = 0
        self.update_time = 0
