#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "Score",
    "ScoreFactor",
    "ScoreThresh",
    ]

from checkbox.lib.enum import Enum


# Score factor values
ScoreFactor = Enum(
    ("UNIQ_MATCH", 100),
    ("CTIME", 1),
    ("INODE", 2),
    ("SAME_SIZE", 2),
    ("GROWN", 1),
    ("SHRUNK", -5),
    )

# Values for min scores
ScoreThresh = Enum(
    ("RESTORE", 10),
    ("FWSEARCH", 4),
    ("NONROT", 3),
    ("MIN_MATCH", 1),
    )


class Score:

    __slots__ = (
        "value",
        )

    def __init__(self, value=None):
        self.value = value

    def __cmp__(self, other):
        if not isinstance(other, Score):
            other = Score(other)

        if not self:
            if not other:
                return 0
            else:
                return -1

        elif not other:
            return 1

        else:
            return cmp(self.value, other.value)

    def __lt__(self, other):
        return self.__cmp__(other) < 0

    def __le__(self, other):
        return self.__cmp__(other) <= 0

    def __gt__(self, other):
        return self.__cmp__(other) > 0

    def __ge__(self, other):
        return self.__cmp__(other) >= 0

    def __eq__(self, other):
        return self.__cmp__(other) == 0

    def __ne__(self, other):
        return self.__cmp__(other) != 0

    def __nonzero__(self):
        return self.value is not None
