#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os
import tempfile

from unittest import TestCase

from checkbox.journal.event import (
    Event,
    EventType,
    )


class EventMixin:

    event_type = None

    def setUp(self):
        self.filename = tempfile.mktemp()

    def tearDown(self):
        if os.path.exists(self.filename):
            os.unlink(self.filename)

    def test_init(self):
        event = Event(self.event_type)
        self.assertEquals(event.type, self.event_type)

    def test_event(self):
        event = Event(self.event_type)
        self.assertFalse(event.putEvent(None))
        self.assertFalse(event.getEvent(None))

    def test_header(self):
        event = Event(self.event_type)
        file = open(self.filename, "w")
        self.assertTrue(event._writeHeader(file))

        file = open(self.filename, "r")
        event_type = int(file.read(3))
        self.assertEquals(event_type, self.event_type)
        self.assertTrue(event._readHeader(file))


class TestGenericEvent(EventMixin, TestCase):

    event_type = EventType.GENERIC

    def test_event(self):
        event = Event(self.event_type)
        event.data = "foo"
        file = open(self.filename, "w")
        self.assertTrue(event.putEvent(file))

        event.data = ""
        file = open(self.filename, "r")
        event_type = int(file.read(3))
        self.assertEquals(event_type, self.event_type)
        self.assertTrue(event.getEvent(file))
        self.assertEquals(event.data, "foo")


class TestExecuteEvent(EventMixin, TestCase):

    event_type = EventType.EXECUTE

    def test_event(self):
        event = Event(self.event_type)
        event.host = "foo.com"
        file = open(self.filename, "w")
        self.assertTrue(event.putEvent(file))

        event.host = ""
        file = open(self.filename, "r")
        event_type = int(file.read(3))
        self.assertEquals(event_type, self.event_type)
        self.assertTrue(event.getEvent(file))
        self.assertEquals(event.host, "foo.com")


class TestSuspendEvent(EventMixin, TestCase):

    event_type = EventType.SUSPEND

    def test_event(self):
        event = Event(self.event_type)
        file = open(self.filename, "w")
        self.assertTrue(event.putEvent(file))

        file = open(self.filename, "r")
        event_type = int(file.read(3))
        self.assertEquals(event_type, self.event_type)
        self.assertTrue(event.getEvent(file))


class TestResumeEvent(EventMixin, TestCase):

    event_type = EventType.RESUME

    def test_event(self):
        event = Event(self.event_type)
        file = open(self.filename, "w")
        self.assertTrue(event.putEvent(file))

        file = open(self.filename, "r")
        event_type = int(file.read(3))
        self.assertEquals(event_type, self.event_type)
        self.assertTrue(event.getEvent(file))


class TestTerminateEvent(EventMixin, TestCase):

    event_type = EventType.TERMINATE

    def test_event(self):
        event = Event(self.event_type)
        file = open(self.filename, "w")
        self.assertTrue(event.putEvent(file))

        file = open(self.filename, "r")
        event_type = int(file.read(3))
        self.assertEquals(event_type, self.event_type)
        self.assertTrue(event.getEvent(file))
