#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os

from unittest import TestCase

from tempfile import mkstemp
from StringIO import StringIO

from checkbox.lib.file import File

from checkbox.journal.event import (
    Event,
    EventType,
    )
from checkbox.journal.writer import Writer


class TestWriter(TestCase):

    def setUp(self):
        (fd, self.path) = mkstemp()
        self.file = File.create_from_fd(fd)

        os.environ["EVENT_JOURNAL"] = self.path

    def tearDown(self):
        if os.path.exists(self.path):
            os.unlink(self.path)

        del os.environ["EVENT_JOURNAL"]

    def test_init(self):
        writer = Writer()

    def test_createFromPath(self):
        writer = Writer.createFromPath()
        writer = Writer.createFromPath(self.path)

    def test_getGlobalPath(self):
        event_log = os.environ.pop("EVENT_JOURNAL")
        writer = Writer.createFromPath()
        self.assertEquals(writer.getGlobalPath(), "")

        os.environ["EVENT_JOURNAL"] = event_log
        writer.configure()
        self.assertEquals(writer.getGlobalPath(), self.path)

    def test_getGlobalSize(self):
        event_log = os.environ.pop("EVENT_JOURNAL")
        writer = Writer.createFromPath()
        self.assertEquals(writer.getGlobalSize(), -1)

        os.environ["EVENT_JOURNAL"] = event_log
        writer.configure()
        self.assertEquals(writer.getGlobalSize(), 0)

    def test_initialize(self):
        writer = Writer()
        self.assertTrue(writer.initialize())

    def test_initializeFromPath(self):
        writer = Writer()
        self.assertFalse(writer.initializeFromPath())
        self.assertTrue(writer.initializeFromPath(self.path))

    def test_writeEvent(self):
        writer = Writer()
        self.assertTrue(writer.writeEvent(None))

        writer.initializeFromPath(self.path)
        self.assertFalse(writer.writeEvent(None))

        event = Event(EventType.GENERIC)
        event.data = "foo"
        self.assertTrue(writer.writeEvent(event))

        self.file.seek(0, os.SEEK_SET)

        # Header event
        line = self.file.readline().strip()
        self.assertTrue("ctime" in line)
        separator = self.file.readline().strip()
        self.assertEquals(separator, "...")

        # Global event
        line = self.file.readline().strip()
        self.assertTrue(line.endswith("foo"))
        separator = self.file.readline().strip()
        self.assertEquals(separator, "...")

        # User event
        line = self.file.readline().strip()
        self.assertTrue(line.endswith("foo"))
        separator = self.file.readline().strip()
        self.assertEquals(separator, "...")

    def test_writeGlobalEvent(self):
        event = Event(EventType.GENERIC)
        event.data = "foo"

        file = StringIO()

        writer = Writer()
        self.assertTrue(writer.writeGlobalEvent(event, file))

        # Does not write header
        file.seek(0, os.SEEK_SET)
        line = file.readline().strip()
        self.assertTrue(line.endswith("foo"))
