#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "Cache",
    "cache",
    ]


class Cache:

    __slots__ = (
        "_function",
        "_cache",
        "_instance",
        )

    def __init__(self, function, cache=dict()):
        self._function = function
        self._cache = cache
        self._instance = None

    def __get__(self, instance, cls=None):
        self._instance = instance
        return self

    def __call__(self, *args):
        key = self._instance and (self._instance,) + args or args
        try:
            return self._cache[key]
        except KeyError:
            value = self._cache[key] = self._function(*key)
            return value


def cache(function):
    return Cache(function)
