#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "CheckException",
    "check_exception",
    "check_key_error",
    ]


class CheckException:

    __slots__ = (
        "_function",
        "_default",
        "_exception",
        "_instance",
        )

    def __init__(self, function, default, exception):
        self._function = function
        self._default = default
        self._exception = exception
        self._instance = None

    def __get__(self, instance, cls=None):
        self._instance = instance
        return self

    def __call__(self, *args, **kwargs):
        try:
            return self._function(self._instance, *args, **kwargs)
        except self.exception:
            return self._default


def check_exception(default=None, exception=Exception):

    def wrapper(func):
        return CheckException(func, default, exception)

    return wrapper


def check_key_error(default=None):
    return check_exception(default, KeyError)
