#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "Enum",
    "EnumError",
    "enum_name_to_value",
    "enum_value_to_names",
    "enum_value_to_name_string",
    ]


class EnumError(Exception):

    pass


class Enum:

    def __init__(self, *names):
        self.__enum_values__ = {}

        value = 0
        unique_names = set()
        for name in names:
            if isinstance(name, (list, tuple)):
                name, value = name

            if not isinstance(name, basestring):
                raise EnumError("enum name is not a string: %s" % name)

            if isinstance(value, basestring):
                if value not in unique_names:
                    raise EnumError("enum value does not define: %s" % value)
                value = getattr(self, value)

            if not isinstance(value, (int, long)):
                raise EnumError("enum value is not an integer: %s" % value)

            if name in unique_names:
                raise EnumError("enum name is not unique: %s" % name)

            # Add name to attributes
            unique_names.add(name)
            setattr(self, name, value)

            # Add value to private dictionary
            values = self.__enum_values__.setdefault(value, [])
            values.append(name)

            value += 1


def enum_name_to_value(enum, name):
    try:
        return getattr(enum, name)
    except AttributeError:
        raise EnumError("enum name unknown: %s" % name)


def enum_value_to_names(enum, value):
    try:
        return enum.__enum_values__[value]
    except KeyError:
        raise EnumError("enum value unknown: %s" % value)


def enum_value_to_name_string(enum, value):
    names = enum_value_to_names(enum, value)
    if len(names) > 2:
        names[:-1] = [", ".join(names[:-1])]

    return " or ".join(names)
