#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "FIFO_BUF",
    "fifo_create",
    ]

import os
import fcntl
import logging

from checkbox.lib.file import (
    fd_open,
    fd_close,
    )


FIFO_BUF = 512


def fifo_create(path, reader, writer):
    # Check that fifo doesn't exist
    if os.path.exists(path):
        os.unlink(path)

    # Create the fifo
    try:
        os.mkfifo(path, 0600)
    except OSError, error:
        logging.info("Failed to create fifo %s: %d", path, error.errno)
        return False

    # Open read side of the fifo initially in non-blocking mode
    read_fd = fd_open(path, os.O_RDONLY | os.O_NONBLOCK)
    if read_fd == -1:
        logging.info("Failed to open read-only %s", path)
        return False

    # Set the fifo back to blocking mode
    flags = fcntl.fcntl(read_fd, fcntl.F_GETFL)
    fcntl.fcntl(read_fd, fcntl.F_SETFL, flags & ~os.O_NONBLOCK)

    # Open write side of the fifo
    write_fd = fd_open(path, os.O_WRONLY)
    if write_fd == -1:
        logging.info("Failed to open write-only %s", path)
        fd_close(read_fd)
        return False

    if (reader.initialize_from_fd(read_fd, "r") or
        writer.initialize_from_fd(write_fd, "w")):
        logging.info("Failed to initialize file descriptors.")
        fd_close(write_fd)
        fd_close(read_fd)
        return False

    # Everything worked
    return True
