#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "hostname",
    "fullhostname",
    "ipaddress",
    ]

from checkbox.lib import (
    net,
    param,
    )
from checkbox.lib.cache import cache


@cache
def hostname():
    hostname = net.gethostname()
    if hostname is None:
        raise Exception("Failed to get host by name")

    return hostname


@cache
def fullhostname():
    return net.getfullhostname(hostname())[0]


@cache
def ipaddress(config_done=False):
    if config_done:
        network_interface = param.get("NETWORK_INTERFACE")
        if network_interface is not None:
            ipaddress, mask_addr = net.is_ipaddress(network_interface)
            if ipaddress is not None:
                return ipaddress

    return net.getfullhostname(fullhostname() or hostname())[1]
