#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.lib.enum import (
    Enum,
    EnumError,
    enum_name_to_value,
    enum_value_to_names,
    enum_value_to_name_string,
    )


class TestEnum(TestCase):

    def test_single(self):
        enum = Enum("FOO")
        self.assertTrue(hasattr(enum, "FOO"))
        self.assertEquals(enum.FOO, 0)

    def test_multiple(self):
        values = (
            "FOO",
            "BAR",
            )
        enum = Enum(*values)
        for i, value in enumerate(values):
            self.assertTrue(hasattr(enum, value))
            self.assertEquals(getattr(enum, value), i)

        self.assertRaises(EnumError, Enum, "FOO", "FOO")

    def test_intKey(self):
        self.assertRaises(EnumError, Enum, 1)

    def test_tupleKey(self):
        int_value = 1
        enum = Enum(("FOO", int_value))
        self.assertEquals(enum.FOO, int_value)

        long_value = 2 ** 40
        enum = Enum(("FOO", long_value))
        self.assertEquals(enum.FOO, long_value)

        enum = Enum("FOO", ("BAR", "FOO"))
        self.assertEquals(enum.BAR, enum.FOO)

        self.assertRaises(EnumError, Enum, ("BAR", "FOO"))

    def test_nameToValue(self):
        enum = Enum("FOO")
        self.assertEquals(enum_name_to_value(enum, "FOO"), 0)

        self.assertRaises(EnumError, enum_name_to_value, enum, "BAR")

    def test_valueToOneName(self):
        enum = Enum("FOO")
        self.assertEquals(enum_value_to_names(enum, 0), ["FOO"])

        self.assertRaises(EnumError, enum_value_to_names, enum, 1)

    def test_valueToTwoNames(self):
        enum = Enum("FOO", ("BAR", "FOO"))
        self.assertEquals(enum_value_to_names(enum, 0), ["FOO", "BAR"])

        self.assertRaises(EnumError, enum_value_to_names, enum, 1)

    def test_valueToOneNameString(self):
        enum = Enum("FOO")
        self.assertEquals(enum_value_to_name_string(enum, 0), "FOO")

    def test_valueToTwoNameString(self):
        enum = Enum("FOO", ("BAR", "FOO"))
        self.assertEquals(enum_value_to_name_string(enum, 0), "FOO or BAR")

    def test_valueToThreeNameString(self):
        enum = Enum("FOO", ("BAR", "FOO"), ("BAZ", "FOO"))
        self.assertEquals(
            enum_value_to_name_string(enum, 0), "FOO, BAR or BAZ")
