#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os

from unittest import TestCase

from tempfile import (
    mkstemp,
    mktemp,
    )

from checkbox.lib.file import (
    File,
    fd_open,
    fd_close,
    )


class TestFile(TestCase):

    def setUp(self):
        self.fd, self.path = mkstemp()
        self.temp = mktemp()

    def tearDown(self):
        if os.path.exists(self.path):
            os.unlink(self.path)

    def test_init(self):
        file = File()
        self.assertFalse(file)

    def test_create_from_path(self):
        file = File.create_from_path(self.path)
        self.assertEquals(file.errno, 0)
        self.assertTrue(file)

        file = File.create_from_path(self.temp)
        self.assertEquals(file.errno, 2)
        self.assertFalse(file)

    def test_initialize_from_fd(self):
        file = File.create_from_fd(self.fd)
        self.assertEquals(file.errno, 0)
        self.assertTrue(file)

        file = File.create_from_fd(-1)
        self.assertEquals(file.errno, 9)
        self.assertFalse(file)

    def test_reset(self):
        file = File.create_from_path(self.path)
        self.assertTrue(file)

        file.reset()
        self.assertFalse(file)

        file = File.create_from_path(self.temp)
        self.assertEquals(file.errno, 2)

        file.reset()
        self.assertEquals(file.errno, 0)

    def test_fd_open(self):
        fd = fd_open(self.path, os.O_RDONLY)
        self.assertTrue(fd >= 0)

        fd = fd_open(self.temp, os.O_RDONLY)
        self.assertEquals(fd, -1)

    def test_fd_close(self):
        ret = fd_close(self.fd)
        self.assertEquals(ret, 0)

        ret = fd_close(-1)
        self.assertEquals(ret, -1)
