#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os

from unittest import TestCase

from tempfile import mkstemp

from checkbox.lib.lock import (
    Lock,
    LockType,
    )


class TestLock(TestCase):

    def setUp(self):
        (self.fd, self.path) = mkstemp()

    def tearDown(self):
        if os.path.exists(self.path):
            os.unlink(self.path)

    def lockChild(self, lock, type):
        blocking = lock.getBlocking()
        pid = os.fork()
        if not pid:
            lock.setBlocking(False)
            os._exit(int(lock.acquire(type)))

        status = os.waitpid(pid, 0)[1]
        lock.setBlocking(blocking)
        return bool(os.WEXITSTATUS(status))

    def test_init(self):
        lock = Lock()
        self.assertTrue(lock.getBlocking())
        self.assertFalse(lock.is_locked)
        self.assertTrue(lock.is_unlocked)
        self.assertEquals(lock.getType(), LockType.UN_LOCK)

    def test_blocking(self):
        lock = Lock()
        self.assertTrue(lock.getBlocking())

        lock.setBlocking(False)
        self.assertFalse(lock.getBlocking())

        lock.setBlocking(True)
        self.assertTrue(lock.getBlocking())

    def test_acquire(self):
        lock = Lock(self.fd)
        self.assertTrue(self.lockChild(lock, LockType.READ_LOCK))
        self.assertTrue(self.lockChild(lock, LockType.WRITE_LOCK))

        self.assertTrue(lock.acquire(LockType.READ_LOCK))
        self.assertTrue(self.lockChild(lock, LockType.READ_LOCK))
        self.assertFalse(self.lockChild(lock, LockType.WRITE_LOCK))

        self.assertTrue(lock.acquire(LockType.WRITE_LOCK))
        self.assertFalse(self.lockChild(lock, LockType.READ_LOCK))
        self.assertFalse(self.lockChild(lock, LockType.WRITE_LOCK))

        self.assertTrue(lock.acquire(LockType.UN_LOCK))
        self.assertTrue(self.lockChild(lock, LockType.READ_LOCK))
        self.assertTrue(self.lockChild(lock, LockType.WRITE_LOCK))

    def test_release(self):
        lock = Lock(self.fd)
        self.assertTrue(lock.acquire(LockType.READ_LOCK))
        self.assertTrue(self.lockChild(lock, LockType.READ_LOCK))
        self.assertFalse(self.lockChild(lock, LockType.WRITE_LOCK))

        self.assertTrue(lock.release())
        self.assertTrue(self.lockChild(lock, LockType.READ_LOCK))
        self.assertTrue(self.lockChild(lock, LockType.WRITE_LOCK))
