#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os

from checkbox.tests.cases import TestCase
from checkbox.tests.env import EnvMixin

from checkbox.lib import param


class TestParam(EnvMixin, TestCase):

    def test_get(self):
        os.environ["FOO"] = "foo"
        self.assertEquals(param.get("FOO"), "foo")

    def test_getBoolean(self):
        for value in "trUE", "yeS", "1":
            os.environ["FOO"] = value
            self.assertEquals(param.getBoolean("FOO", False), True)

        for value in "FALSE", "No", "0":
            os.environ["FOO"] = value
            self.assertEquals(param.getBoolean("FOO", True), False)

        del os.environ["FOO"]
        self.assertEquals(param.getBoolean("FOO", True), True)

    def test_getInteger(self):
        for integer in "0", "1", "100":
            os.environ["FOO"] = integer
            self.assertEquals(param.getInteger("FOO", -1), int(integer))

        del os.environ["FOO"]
        self.assertEquals(param.getInteger("FOO", -1), -1)
