#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import signal

from unittest import TestCase

from checkbox.lib.sigprocmask import (
    SIG_BLOCK,
    SIG_SETMASK,
    SIG_UNBLOCK,
    sigprocmask,
    )
from checkbox.lib.sigset import Sigset


class TestSigprocmask(TestCase):

    def tearDown(self):
        old = sigprocmask(0)
        sigprocmask(SIG_UNBLOCK, old)

    def test_noset(self):
        old = sigprocmask(0)
        self.assertEquals(len(old), 0)

    def test_how(self):
        # This shouldn't raise an exception
        sigprocmask(3)

        new = Sigset()
        self.assertRaises(OSError, sigprocmask, 3, new)

    def test_set(self):
        # This shouldn't raise an exception
        new = Sigset([signal.NSIG - 1])
        sigprocmask(0, new)

        new = Sigset([signal.NSIG])
        self.assertRaises(OSError, sigprocmask, 0, new)

    def test_setmask(self):
        new = Sigset([signal.SIGUSR1])
        old = sigprocmask(SIG_SETMASK, new)
        self.assertEquals(len(old), 0)

        old = sigprocmask(0)
        self.assertEquals(old, new)

    def test_block(self):
        new = Sigset([signal.SIGUSR1])
        sigprocmask(SIG_BLOCK, new)

        new = Sigset([signal.SIGUSR2])
        sigprocmask(SIG_BLOCK, new)

        new = Sigset([signal.SIGUSR1, signal.SIGUSR2])
        old = sigprocmask(0)
        self.assertEquals(old, new)

    def test_unblock(self):
        new = Sigset([signal.SIGUSR1])
        sigprocmask(SIG_BLOCK, new)
        sigprocmask(SIG_UNBLOCK, new)

        new = Sigset()
        old = sigprocmask(0)
        self.assertEquals(old, new)
