#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os

from unittest import TestCase

from tempfile import (
    mktemp,
    mkstemp,
    )

from checkbox.lib.stat import Stat


class TestStat(TestCase):

    def setUp(self):
        self.not_exists_path = mktemp()
        self.exists_path = mkstemp()[1]

    def tearDown(self):
        if os.path.exists(self.exists_path):
            os.unlink(self.exists_path)

    def test_init(self):
        stat = Stat()
        self.assertFalse(stat.is_valid())
        self.assertFalse(stat.is_result_valid())
        self.assertEquals(stat.get_result(), None)
        self.assertEquals(stat.get_status(), 0)
        self.assertEquals(stat.get_errno(), 0)

    def test_path(self):
        stat = Stat()
        self.assertTrue(stat.set_path(None))
        self.assertFalse(stat.is_valid())

        self.assertEquals(stat.stat_path(None), -3)
        self.assertEquals(stat.get_errno(), 0)

        self.assertTrue(stat.set_path(self.not_exists_path))
        self.assertTrue(stat.is_valid())

        self.assertEquals(stat.stat_path(self.not_exists_path), -1)
        self.assertEquals(stat.get_errno(), 2)

        self.assertTrue(stat.set_path(self.exists_path))
        self.assertTrue(stat.is_valid())

        self.assertEquals(stat.stat_path(self.exists_path), 0)
        self.assertEquals(stat.get_errno(), 0)

    def test_fd(self):
        stat = Stat()
        self.assertTrue(stat.set_fd(-1))
        self.assertFalse(stat.is_valid())

        self.assertEquals(stat.stat_fd(-1), -3)
        self.assertEquals(stat.get_errno(), 0)

        self.assertTrue(stat.set_fd(0))
        self.assertTrue(stat.is_valid())

        self.assertEquals(stat.stat_fd(0), 0)
        self.assertEquals(stat.get_errno(), 0)

    def test_result(self):
        stat = Stat()
        self.assertEquals(stat.is_result_valid(), False)
        self.assertEquals(stat.get_result(), None)

        stat.stat_path(self.not_exists_path)
        self.assertEquals(stat.is_result_valid(), False)
        self.assertNotEquals(stat.get_result(), None)
        self.assertEquals(stat.get_result().st_ino, 0)

        stat.stat_path(self.exists_path)
        self.assertEquals(stat.is_result_valid(), True)
        self.assertNotEquals(stat.get_result(), None)
        self.assertNotEquals(stat.get_result().st_ino, 0)
