#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "convert_escapes",
    "unconvert_escapes",
    ]

from string import (
    octdigits,
    printable,
    )


ESCAPE_TO_CHAR = {
    "a": "\a",
    "b": "\b",
    "f": "\f",
    "n": "\n",
    "r": "\r",
    "t": "\t",
    "v": "\v",
    "\\": "\\",
    "\'": "\'",
    "\"": "\"",
    "?": "?"}

CHAR_TO_ESCAPE = dict((v, k) for k, v in ESCAPE_TO_CHAR.items())


def convert_escapes(text):
    copy = ""
    source = 0
    length = len(text)
    while source < length:
        if text[source] != "\\" or source == length - 1:
            copy += text[source]

        else:
            source += 1
            if text[source] in ESCAPE_TO_CHAR:
                copy += ESCAPE_TO_CHAR[text[source]]

            elif text[source] in octdigits:
                number = None
                if (source <= length - 3 and
                    text[source] >= "0" and text[source] <= "3" and
                    text[source + 1] in octdigits and
                    text[source + 2] in octdigits):
                    number = int(text[source:source + 3], 8)
                    copy += chr(number)
                    source += 2

                elif (source <= length - 2 and
                    text[source + 1] in octdigits):
                    number = int(text[source:source + 2], 8)
                    copy += chr(number)
                    source += 1

                elif source <= length - 1:
                    number = int(text[source:source + 1], 8)
                    copy += chr(number)

                else:
                    copy += text[source]

                if number == 0:
                    return None

            else:
                copy += text[source - 1]
                copy += text[source]

        source += 1

    return copy


def unconvert_escapes(text, delimiter=None):
    copy = ""

    for c in text:
        if c == delimiter:
            copy += "\\%s" % c

        elif c in CHAR_TO_ESCAPE:
            copy += "\\%s" % CHAR_TO_ESCAPE[c]

        elif c not in printable:
            copy += "\\%03o" % ord(c)

        else:
            copy += c

    return copy
