#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "Expression",
    ]

from checkbox.message.eval import EvalState


class Expression:

    __slots__ = (
        "parent",
        )

    def __init__(self):
        self.parent = None

    def __eq__(self, other):
        return self.sameAs(other)

    def __ne__(self, other):
        return not self.sameAs(other)

    def copy(self):
        other = object.__new__(self.__class__)
        other.copyFrom(self)
        return other

    def copyFrom(self, other):
        self.parent = other.parent

    def sameAs(self, other):
        if other is None or type(other) != type(self):
            return False

        return True

    def evaluate(self, state=None):
        if state is None:
            state = EvalState()
            state.setScope(self.parent)

        return self._evaluate(state)

    def _evaluate(self, state):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def setParentScope(self, parent):
        self.parent = parent
        self._setParentScope(parent)

    def _setParentScope(self, parent):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def flatten(self, state, kind=None):
        return self._flatten(state, kind)[0:2]

    def _flatten(self, state, kind):
        raise NotImplementedError(
            "this function must be overridden by subclasses")
