#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "List",
    ]

from checkbox.message.expression import Expression
from checkbox.message.literal import Literal
from checkbox.message.types import undefined


class List(Expression):

    __slots__ = (
        "expressions",
        )

    def __init__(self, expressions=[]):
        super(List, self).__init__()
        self.expressions = list(expressions)

    def __getattr__(self, name):
        return getattr(self.expressions, name)

    def __iter__(self):
        return iter(self.expressions)

    def __len__(self):
        return len(self.expressions)

    def copyFrom(self, other):
        super(List, self).copyFrom(other)

        self.expressions = []
        for expression in other.expressions:
            self.expressions.append(expression.copy())

    def sameAs(self, other):
        if not super(List, self).sameAs(other):
            return False

        if len(self) != len(other):
            return False

        for s, o in zip(self.expressions, other.expressions):
            if not s.sameAs(o):
                return False

        return True

    def _setParentScope(self, parent):
        for expression in self.expressions:
            expression.setParentScope(parent)

    def _evaluate(self, state):
        return self.expressions

    def _evaluateTree(self, state, tree=None):
        value = self._evaluate(state)
        return (value, self)

    def _flatten(self, state, kind):
        expr_list = List()

        for expression in self.expressions:
            value, tree = expression.flatten(state)
            if not tree:
                tree = Literal(value)

            expr_list.append(tree)

        return (undefined, expr_list, kind)
