#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "Literal",
    ]

from copy import copy

from checkbox.lib.enum import Enum

from checkbox.message.expression import Expression
from checkbox.message.types import undefined


LiteralFactor = Enum(
    ("NONE", 0),        # No factor specified
    ("BYTE", 2 ** 0),   # Byte factor
    ("KILO", 2 ** 10),  # Kilo factor
    ("MEGA", 2 ** 20),  # Mega factor
    ("GIGA", 2 ** 30),  # Giga factor
    ("TERA", 2 ** 40),  # Terra factor
    )


class Literal(Expression):

    __slots__ = (
        "value",
        "factor",
        )

    def __init__(self, value=undefined, factor=LiteralFactor.NONE):
        super(Literal, self).__init__()
        if isinstance(value, dict) or isinstance(value, list):
            raise Exception("list and dict values are not literals")

        self.value = value
        self.factor = factor

    def _setParentScope(self, parent):
        # Already set by base class for this node
        pass

    def copyFrom(self, other):
        super(Literal, self).copyFrom(other)
        self.value = copy(other.value)
        self.factor = other.factor

    def sameAs(self, other):
        if not super(Literal, self).sameAs(other):
            return False

        if (self.factor != other.factor or
            self.value != other.value):
            return False

        return True

    def _evaluate(self, state):
        value = self.value

        if (self.factor and
            (isinstance(value, int) or isinstance(value, float))):

            value = float(value * self.factor)

        return value

    def _evaluateTree(self, state, tree):
        value = self._evaluate(state)

        return (value, self)

    def _flatten(self, state, kind):
        value = self._evaluate(state)
        return (value, None, kind)
