#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.message.convert import convert_escapes


class TestConvert(TestCase):

    def test_escapes(self):
        escape_table = (
            ("\a", "\\a"),
            ("\b", "\\b"),
            ("\f", "\\f"),
            ("\n", "\\n"),
            ("\r", "\\r"),
            ("\t", "\\t"),
            ("\v", "\\v"),
            ("\\", "\\"),
            ("\'", "\'"),
            ("\"", "\""),
            ("?", "\\?"))
        for k, v in escape_table:
            self.assertEquals(k, convert_escapes(v))

    def test_strings(self):
        string_table = (
            "",
            "i",
            "ij",
            "\\",
            "i\\",
            "\\i")
        for s in string_table:
            self.assertEquals(s, convert_escapes(s))

    def test_octals(self):
        octal_table = (
            ("\t", "\\11"),
            ("\t", "\\011"),
            ("I", "\\111"))

        for k, v in octal_table:
            self.assertEquals(k, convert_escapes(v))

    def test_null(self):
        self.assertEquals(convert_escapes("\\0"), None)
