#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.message.lexer import Lexer
from checkbox.message.token import (
    Token,
    TokenType,
    )


class TestLexer(TestCase):

    def setUp(self):
        self.lexer = Lexer()

    def test_peekToken(self):
        self.lexer.initialize("1")

        token_type = self.lexer.peekToken()
        self.assertEquals(token_type, TokenType.INTEGER)

        token_type = self.lexer.peekToken()
        self.assertEquals(token_type, TokenType.INTEGER)

    def test_consumeToken(self):
        self.lexer.initialize("1")

        peek = self.lexer.peekToken()
        consume = self.lexer.consumeToken()
        self.assertEquals(peek, consume)

        peek = self.lexer.peekToken()
        self.assertEquals(peek, TokenType.END_OF_INPUT)

    def test_multiple(self):
        self.lexer.initialize("1 * 2")
        token = Token()

        token_type = self.lexer.consumeToken(token)
        self.assertEquals(token_type, TokenType.INTEGER)
        self.assertEquals(token.value, 1)

        token_type = self.lexer.consumeToken()
        self.assertEquals(token_type, TokenType.MULTIPLY)

        token_type = self.lexer.consumeToken(token)
        self.assertEquals(token_type, TokenType.INTEGER)
        self.assertEquals(token.value, 2)

    def test_brace(self):
        self.lexer.initialize("{ 1 }")
        token = Token()

        token_type = self.lexer.consumeToken()
        self.assertEquals(token_type, TokenType.OPEN_BRACE)

        token_type = self.lexer.consumeToken(token)
        self.assertEquals(token_type, TokenType.INTEGER)
        self.assertEquals(token.value, 1)

        token_type = self.lexer.consumeToken()
        self.assertEquals(token_type, TokenType.CLOSE_BRACE)

    def test_identifier(self):
        self.lexer.initialize("identifier1")
        token = Token()

        token_type = self.lexer.consumeToken(token)
        self.assertEquals(token_type, TokenType.IDENTIFIER)
        self.assertEquals(token.value, "identifier1")

        self.lexer.initialize("'identifier2'")
        token = Token()

        token_type = self.lexer.consumeToken(token)
        self.assertEquals(token_type, TokenType.IDENTIFIER)
        self.assertEquals(token.value, "identifier2")

    def test_string(self):
        self.lexer.initialize('"string"')
        token = Token()

        token_type = self.lexer.consumeToken(token)
        self.assertEquals(token_type, TokenType.STRING)
        self.assertEquals(token.value, "string")
