#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.message.eval import EvalState
from checkbox.message.function import Function
from checkbox.message.list import List
from checkbox.message.literal import Literal
from checkbox.message.record import Record
from checkbox.message.reference import Reference
from checkbox.message.types import undefined


class TestList(TestCase):

    def test_copyFrom(self):
        literal1 = Literal(1)
        literal2 = Literal(2)

        # Different lists
        list1 = List([literal1])
        list2 = List([literal2])
        self.assertNotEquals(list1, list2)

        # Same lists
        list1.copyFrom(list2)
        self.assertEquals(list1, list2)

        # Change list
        literal2.value = 3
        self.assertNotEquals(list1, list2)

    def test_sameAs(self):
        # Same reference
        list1 = List()
        self.assertNotEquals(list1, None)
        self.assertEquals(list1, list1)

        # Different reference
        list2 = List()
        self.assertEquals(list1, list2)

        # Same literals
        literal1 = Literal(1)
        list1 = List([literal1])
        list2 = List([literal1])
        self.assertEquals(list1, list2)

        # Same literal data
        literal2 = Literal(1)
        list1 = List([literal1])
        list2 = List([literal2])
        self.assertEquals(list1, list2)

        # Same keys, different literal data
        literal2 = Literal(2)
        list1 = List([literal1])
        list2 = List([literal2])
        self.assertNotEquals(list1, list2)

    def test_evaluate(self):
        expressions = [Literal(1)]
        expression = List(expressions)

        result = expression.evaluate()
        self.assertTrue(isinstance(result, list))
        self.assertEquals(result, expressions)

    def test_flatten(self):
        expressions = [Literal(1)]
        expression = List(expressions)
        state = EvalState()
        value, tree = expression.flatten(state)

        self.assertEquals(value, undefined)
        self.assertTrue(isinstance(tree, List))
        self.assertTrue(tree.expressions, expressions)

    def test_flatten_function(self):
        expression = List([Function("int", Literal("1"))])
        state = EvalState()
        value, tree = expression.flatten(state)

        self.assertEquals(value, undefined)
        self.assertTrue(isinstance(tree, List))
        self.assertTrue(tree.expressions[0], Literal(1))

    def test_flatten_reference(self):
        expression = List([Reference("a")])
        state = EvalState()
        state.setScope(Record({"a": Literal(1)}))
        value, tree = expression.flatten(state)

        self.assertEquals(value, undefined)
        self.assertTrue(isinstance(tree, List))
        self.assertTrue(tree.expressions[0], Literal(1))
