#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.message.eval import EvalState
from checkbox.message.literal import (
    Literal,
    LiteralFactor,
    )


class TestLiteral(TestCase):

    def test_copyFrom(self):
        # Different literals
        literal1 = Literal("a")
        literal2 = Literal("b")
        self.assertNotEquals(literal1, literal2)

        # Same literals
        literal1.copyFrom(literal2)
        self.assertEquals(literal1, literal2)

        # Change literal
        literal1.value = "a"
        self.assertNotEquals(literal1, literal2)

    def test_sameAs(self):
        # Same reference
        literal1 = Literal()
        self.assertNotEquals(literal1, None)
        self.assertEquals(literal1, literal1)

        # Different reference
        literal2 = Literal()
        self.assertEquals(literal1, literal2)

        # Same value reference
        literal1 = Literal("a")
        literal2 = Literal("a")
        self.assertEquals(literal1, literal2)

        # Different value reference, same data
        literal2 = Literal("a")
        self.assertEquals(literal1, literal2)

        # Different value reference, different data
        literal2 = Literal("b")
        self.assertNotEquals(literal1, literal2)

    def test_evaluate_boolean(self):
        literal = Literal(True)

        result = literal.evaluate()
        self.assertTrue(isinstance(result, bool))
        self.assertEquals(result, True)

    def test_evaluate_integer(self):
        literal = Literal(1)

        result = literal.evaluate()
        self.assertTrue(isinstance(result, int))
        self.assertEquals(result, 1)

    def test_evaluate_integer_with_factor(self):
        literal = Literal(1, LiteralFactor.KILO)

        result = literal.evaluate()
        self.assertTrue(isinstance(result, float))
        self.assertEquals(result, 2.0 ** 10)

    def test_evaluate_float(self):
        literal = Literal(1.1)

        result = literal.evaluate()
        self.assertTrue(isinstance(result, float))
        self.assertEquals(result, 1.1)

    def test_evaluate_float_with_factor(self):
        literal = Literal(1.1, LiteralFactor.KILO)

        result = literal.evaluate()
        self.assertTrue(isinstance(result, float))
        self.assertEquals(result, 1126.4)

    def test_evaluate_string(self):
        literal = Literal("a")

        result = literal.evaluate()
        self.assertTrue(isinstance(result, basestring))
        self.assertEquals(result, "a")

    def test_flatten(self):
        literal = Literal(1)
        state = EvalState()
        value, tree = literal.flatten(state)
        self.assertEquals(value, 1)
        self.assertEquals(tree, None)
