#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.message.parser import (
    Parser,
    ParserError,
    )
from checkbox.message.token import TokenType


class TestParserError(TestCase):

    def test_no_arg(self):
        error = ParserError(TokenType.COLON)
        self.assertEquals(str(error), "Expected nothing but got COLON")

    def test_one_arg(self):
        error = ParserError(TokenType.COLON, TokenType.BOOLEAN)
        self.assertEquals(str(error), "Expected BOOLEAN but got COLON")

    def test_two_args(self):
        error = ParserError(
            TokenType.COLON, TokenType.BOOLEAN, TokenType.INTEGER)
        self.assertEquals(
            str(error), "Expected BOOLEAN or INTEGER but got COLON")

    def test_three_args(self):
        error = ParserError(
            TokenType.COLON, TokenType.BOOLEAN, TokenType.INTEGER,
            TokenType.REAL)
        self.assertEquals(
            str(error), "Expected BOOLEAN, INTEGER or REAL but got COLON")


class TestParser(TestCase):

    def setUp(self):
        self.parser = Parser()

    def test_parseExpression(self):
        tree = self.parser.parseExpression("true ? 1 : 2")
        self.assertNotEquals(tree, None)
        value = tree.evaluate()
        self.assertEquals(value, 1)

        tree = self.parser.parseExpression("false ? 1 : 2")
        self.assertNotEquals(tree, None)
        value = tree.evaluate()
        self.assertEquals(value, 2)

    def test_parseRecord(self):
        tree = self.parser.parseExpression("{ a = 1 }")
        self.assertNotEquals(tree, None)
        self.assertTrue("a" in tree.references)
        self.assertEquals(tree.references["a"].value, 1)

        tree = self.parser.parseExpression("{ a = 1; b = 2 }")
        self.assertNotEquals(tree, None)
        self.assertTrue("a" in tree.references and "b" in tree.references)
        self.assertEquals(tree.references["a"].value, 1)
        self.assertEquals(tree.references["b"].value, 2)

    def test_parseReference(self):
        tree = self.parser.parseExpression("{ a = 1; b = a }")
        self.assertNotEquals(tree, None)
        self.assertTrue("a" in tree.references and "b" in tree.references)
        self.assertEquals(tree.references["a"].value, 1)

        value = tree.references["b"].evaluate()
        self.assertEquals(value, 1)

    def test_parseFunction(self):
        tree = self.parser.parseExpression("int(1)")
        self.assertNotEquals(tree, None)

        value = tree.evaluate()
        self.assertTrue(isinstance(value, int))
        self.assertEquals(value, 1)

        tree = self.parser.parseExpression("int(\"1\")")
        value = tree.evaluate()
        self.assertTrue(isinstance(value, int))
        self.assertEquals(value, 1)
