#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.message.literal import Literal
from checkbox.message.record import Record


class TestRecord(TestCase):

    def test_copyFrom(self):
        literal1 = Literal(1)
        literal2 = Literal(2)

        # Different records
        record1 = Record({"a": literal1})
        record2 = Record({"b": literal2})
        self.assertNotEquals(record1, record2)

        # Same records
        record1.copyFrom(record2)
        self.assertEquals(record1, record2)

    def test_sameAs(self):
        # Same reference
        record1 = Record()
        self.assertNotEquals(record1, None)
        self.assertEquals(record1, record1)

        # Different reference
        record2 = Record()
        self.assertEquals(record1, record2)

        # Same keys and literals
        literal1 = Literal(1)
        record1 = Record({"a": literal1})
        record2 = Record({"a": literal1})
        self.assertEquals(record1, record2)

        # Different keys, same literals
        record1 = Record({"a": literal1})
        record2 = Record({"b": literal1})
        self.assertNotEquals(record1, record2)

        # Same keys, same literal data
        literal2 = Literal(1)
        record1 = Record({"a": literal1})
        record2 = Record({"a": literal2})
        self.assertEquals(record1, record2)

        # Same keys, different literal data
        literal2 = Literal(2)
        record1 = Record({"a": literal1})
        record2 = Record({"a": literal2})
        self.assertNotEquals(record1, record2)

    def test_evaluate(self):
        references = {"a": Literal(1)}
        record = Record(references)

        result = record.evaluate()
        self.assertTrue(isinstance(result, dict))
        self.assertEquals(result, references)

    def test_evaluateName(self):
        literal = Literal(1)
        record = Record({"a": literal})

        result = record.evaluateName("a")
        self.assertTrue(isinstance(result, int))
        self.assertEquals(result, 1)

    def test_lookup(self):
        literal = Literal(1)
        record = Record({"a": literal})

        tree = record.lookup("b")
        self.assertFalse(tree)

        tree = record.lookup("a")
        self.assertTrue(tree)
        self.assertEquals(tree.value, 1)
