#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.message.eval import EvalState
from checkbox.message.literal import Literal
from checkbox.message.record import Record
from checkbox.message.reference import Reference
from checkbox.message.types import Undefined


class TestReference(TestCase):

    def test_copyFrom(self):
        literal1 = Literal("a")
        literal2 = Literal("b")

        # Different references
        reference1 = Reference("a", True, literal1)
        reference2 = Reference("b", False, literal2)
        self.assertNotEquals(reference1, reference2)

        # Same references
        reference1.copyFrom(reference2)
        self.assertEquals(reference1, reference2)

        # Change reference
        reference1.name = "a"
        self.assertNotEquals(reference1, reference2)

    def test_sameAs(self):
        # Same reference
        reference1 = Reference()
        self.assertNotEquals(reference1, None)
        self.assertEquals(reference1, reference1)

        # Different reference
        reference2 = Reference()
        self.assertEquals(reference1, reference2)

        # Same name
        reference1 = Reference("a")
        reference2 = Reference("a")
        self.assertEquals(reference1, reference2)

        # Different name
        reference1 = Reference("a")
        reference2 = Reference("b")
        self.assertNotEquals(reference1, reference2)

        # Same name, different absolute
        reference1 = Reference("a", True)
        reference2 = Reference("a", False)
        self.assertNotEquals(reference1, reference2)

        # Same name, same expression reference
        literal1 = Literal("a")
        reference1 = Reference("a", True, literal1)
        reference2 = Reference("a", True, literal1)
        self.assertEquals(reference1, reference2)

        # Same name, same expression value
        literal2 = Literal("a")
        reference2 = Reference("a", True, literal2)
        self.assertEquals(reference1, reference2)

        # Same name, different expression
        literal2 = Literal("b")
        reference2 = Reference("a", True, literal2)
        self.assertNotEquals(reference1, reference2)

        # Different name, different expression
        reference2 = Reference("b", True, literal2)
        self.assertNotEquals(reference1, reference2)

    def test_evaluate(self):
        # Without state
        reference = Reference("a")
        result = reference.evaluate()
        self.assertTrue(isinstance(result, Undefined))

        # With existent state
        literal = Literal(1)
        record = Record({"a": literal})
        state = EvalState()
        state.setScope(record)
        result = reference.evaluate(state)
        self.assertTrue(isinstance(result, int))
        self.assertEquals(result, 1)

        # With non-existent state
        record = Record({"b": literal})
        state.setScope(record)
        result = reference.evaluate(state)
        self.assertTrue(isinstance(result, Undefined))
