#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "Token",
    "TokenType",
    ]

from checkbox.lib.enum import Enum

from checkbox.message.literal import LiteralFactor


TokenType = Enum(
    "TOKEN_ERROR",
    "END_OF_INPUT",
    "TOKEN_TOO_LONG",
    "INTEGER",
    "REAL",
    "BOOLEAN",
    "STRING",
    "NULL",
    "UNDEFINED",
    "ERROR",
    "IDENTIFIER",
    "SELECTION",
    "MULTIPLY",
    "DIVIDE",
    "MODULUS",
    "PLUS",
    "MINUS",
    "BITWISE_AND",
    "BITWISE_OR",
    "BITWISE_NOT",
    "BITWISE_XOR",
    "LEFT_SHIFT",
    "RIGHT_SHIFT",
    "URIGHT_SHIFT",
    "LOGICAL_AND",
    "LOGICAL_OR",
    "LOGICAL_NOT",
    "IN",
    "LESS_THAN",
    "LESS_OR_EQUAL",
    "GREATER_THAN",
    "GREATER_OR_EQUAL",
    "EQUAL",
    "NOT_EQUAL",
    "META_EQUAL",
    "META_NOT_EQUAL",
    "BOUND_TO",
    "QMARK",
    "COLON",
    "COMMA",
    "SEMICOLON",
    "NEW_LINE",
    "NEW_PARAGRAPH",
    "OPEN_BRACKET",
    "CLOSE_BRACKET",
    "OPEN_PAREN",
    "CLOSE_PAREN",
    "OPEN_BRACE",
    "CLOSE_BRACE",
    "BACKSLASH",
    )


class Token:

    __slots__ = (
        "factor",
        "type",
        "value",
        )

    def __init__(self):
        self.factor = LiteralFactor.NONE
        self.type = TokenType.TOKEN_ERROR
        self.value = None

    def copyFrom(self, other):
        self.factor = other.factor
        self.type = other.type
        self.value = other.value
