#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "Env",
    ]

import logging

from checkbox.process.args import Args


NO_ENV_VALUE = "\01\02\03\04\05\06"


class Env(dict):

    def __init__(self, **kwargs):
        self.update(kwargs)

    def set(self, name, value):
        if not name:
            return False

        success = name not in self
        self[name] = value

        return success

    def setString(self, name_value):
        if not name_value:
            return False

        delimiter = name_value.find("=")
        if delimiter == -1 and "$$" in name_value:
            self.set(name_value, NO_ENV_VALUE)
            return True

        if delimiter == -1:
            logging.error(
                "Missing '=' after environment variable: %s", name_value)
            return False

        elif delimiter == 0:
            logging.error(
                "Missing environment variable: %s", name_value)
            return False

        return self.set(name_value[:delimiter], name_value[delimiter + 1:])

    def mergeEnv(self, env):
        self.update(env)

    def mergeQuoted(self, quoted):
        raw = self.quotedToRaw(quoted)
        if raw is None:
            return False

        return self.mergeRaw(raw)

    def mergeRaw(self, raw):
        env_list = []
        Args.splitRaw(raw, env_list)

        for env in env_list:
            if not self.setString(env):
                return False

        return True

    def mergeRecord(self, record, attribute):
        string = record.evaluateString(attribute)
        if string is not None:
            return self.mergeRaw(string)

        else:
            return True

    @classmethod
    def quotedToRaw(cls, quoted):
        return Args.quotedToRaw(quoted)
