#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "Manager",
    "create_manager",
    ]

from checkbox.lib import param


class ManagerError(Exception):

    pass


class Manager:

    @property
    def is_direct(self):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def registerSubfamily(self, pid, watcher_pid, interval):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def trackEnvironment(self, pid, pid_env_id):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def trackLogin(self, pid, login):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def signalProcess(self, pid, signal):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def suspendFamily(self, pid):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def resumeFamily(self, pid):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def killFamily(self, pid):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def unregisterFamily(self, pid):
        raise NotImplementedError(
            "this function must be overridden by subclasses")


def create_manager(subsystem=None):
    is_master = subsystem == "MASTER"
    if param.getBoolean("USE_PROCESS_SERVER", not is_master):
        from checkbox.process.managerindirect import ManagerIndirect

        suffix = None if is_master else subsystem
        return ManagerIndirect(suffix)
    else:
        from checkbox.process.managerdirect import ManagerDirect

        return ManagerDirect()
