#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "ManagerMixin",
    ]

import signal

from checkbox.process.tests.drone import DroneMixin


class ManagerMixin(DroneMixin):

    def makeManager(self):
        raise NotImplementedError(
            "this function must be overridden by subclasses")

    def test_registerSubfamily(self):
        pid = self.spawn()
        manager = self.makeManager()
        self.assertTrue(manager.registerSubfamily(pid, 0, -1))

    def test_signalProcess(self):
        pid = self.spawn()
        manager = self.makeManager()
        self.assertFalse(manager.signalProcess(pid, signal.SIGUSR1))
        manager.registerSubfamily(pid, 0, -1)
        self.assertTrue(manager.signalProcess(pid, signal.SIGUSR1))

    def test_unregisterFamily(self):
        pid = self.spawn()
        manager = self.makeManager()
        self.assertFalse(manager.unregisterFamily(pid))
        manager.registerSubfamily(pid, 0, -1)
        self.assertTrue(manager.unregisterFamily(pid))
