#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.process.args import Args


class TestArgs(TestCase):

    def test_len(self):
        args = Args()
        self.assertEquals(len(args), 0)

        args = Args("1")
        self.assertEquals(len(args), 1)

    def test_iter(self):
        args = Args("1")
        for arg in args:
            self.assertEquals(arg, "1")

        args = Args("1", "1")
        for arg in args:
            self.assertEquals(arg, "1")

    def test_index(self):
        args = Args("1", "2")
        self.assertEquals(args[0], "1")
        self.assertEquals(args[1], "2")

    def test_append(self):
        args = Args()
        self.assertEquals(len(args), 0)

        args.append("1")
        self.assertEquals(len(args), 1)

    def test_quotedToRaw(self):
        self.assertEquals(Args.quotedToRaw('"1"'), "1")
        self.assertEquals(Args.quotedToRaw('"1 2"'), "1 2")

        self.assertEquals(Args.quotedToRaw('  "1"'), "1")
        self.assertEquals(Args.quotedToRaw('"1"  '), "1")

    def test_argToRaw(self):
        self.assertEquals(Args.argToRaw("1"), "1")
        self.assertEquals(Args.argToRaw("1 2"), "1' '2")

    def test_joinArgs(self):
        self.assertEquals(Args.joinArgs(["1"]), "1")
        self.assertEquals(Args.joinArgs(["1", "2"]), "1 2")
        self.assertEquals(Args.joinArgs(["1", "2"], 1), "2")

    def test_splitRaw(self):
        args = []
        self.assertTrue(Args.splitRaw("1", args))
        self.assertEquals(len(args), 1)

        args = Args()
        self.assertTrue(Args.splitRaw("1 2", args))
        self.assertEquals(len(args), 2)

        args = []
        self.assertTrue(Args.splitRaw("1 '2'", args))
        self.assertEquals(len(args), 2)
        self.assertEquals(args[0], "1")
        self.assertEquals(args[1], "2")

        args = []
        self.assertTrue(Args.splitRaw("''", args))
        self.assertEquals(len(args), 1)
        self.assertEquals(args[0], "")

        args = []
        self.assertTrue(Args.splitRaw("'foo''s'", args))
        self.assertEquals(len(args), 1)
        self.assertEquals(args[0], "foo's")
