#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.process.env import Env


class TestEnv(TestCase):

    def test_len(self):
        env = Env()
        self.assertEquals(len(env), 0)

        env = Env(a="1")
        self.assertEquals(len(env), 1)

    def test_items(self):
        env = Env(a="1")
        for k, v in env.items():
            self.assertEquals(k, "a")
            self.assertEquals(v, "1")

        for k in env.keys():
            self.assertEquals(k, "a")

        for v in env.values():
            self.assertEquals(v, "1")

    def test_getitem(self):
        env = Env(a="1")
        self.assertEquals(env["a"], "1")

    def test_set(self):
        env = Env()
        self.assertEquals(len(env), 0)

        self.assertTrue(env.set("a", "1"))
        self.assertEquals(env["a"], "1")

    def test_setString(self):
        env = Env()
        self.assertEquals(len(env), 0)

        self.assertTrue(env.setString("a=1"))
        self.assertEquals(env["a"], "1")

    def test_mergeQuoted(self):
        env = Env()
        self.assertTrue(env.mergeQuoted('"a=1"'))
        self.assertEquals(env["a"], "1")

        env = Env()
        self.assertTrue(env.mergeQuoted("\"a=1 b='2'\""))
        self.assertEquals(env["a"], "1")
        self.assertEquals(env["b"], "2")

    def test_mergeRaw(self):
        env = Env()
        self.assertTrue(env.mergeRaw('a=1'))
        self.assertEquals(env["a"], "1")

        env = Env()
        self.assertTrue(env.mergeRaw("a=1 b='2'"))
        self.assertEquals(env["a"], "1")
        self.assertEquals(env["b"], "2")
