#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.process.pidenv import (
    PidEnvIdStatus,
    PidEnvIdEntry,
    PidEnvId,
    )


class TestPidEnvId(TestCase):

    def test_cmp(self):
        pid_env_id1 = PidEnvId()
        self.assertNotEquals(pid_env_id1, None)

        pid_env_id2 = PidEnvId()
        self.assertNotEquals(pid_env_id1, pid_env_id2)

        pid_env_id1.append("foo=bar")
        self.assertNotEquals(pid_env_id1, pid_env_id2)

        pid_env_id2.append("foo=bar")
        self.assertEquals(pid_env_id1, pid_env_id2)

        pid_env_id1.append("bar=baz")
        self.assertNotEquals(pid_env_id1, pid_env_id2)
        self.assertEquals(pid_env_id2, pid_env_id1)

    def test_copy(self):
        pid_env_id1 = PidEnvId()
        pid_env_id1.append("foo=bar")

        pid_env_id2 = pid_env_id1.copy()
        self.assertEquals(pid_env_id1, pid_env_id2)

    def test_copy_from(self):
        pid_env_id1 = PidEnvId()
        pid_env_id1.append("foo=bar")

        pid_env_id2 = PidEnvId()
        self.assertNotEquals(pid_env_id1, pid_env_id2)

        pid_env_id2.copy_from(pid_env_id1)
        self.assertEquals(pid_env_id1, pid_env_id2)

    def test_append(self):
        pid_env_id = PidEnvId()
        pid_env_id.append("foo=bar")
        self.assertEquals(pid_env_id.ancestors[0].envid, "foo=bar")
        self.assertTrue(pid_env_id.ancestors[0].active)
        self.assertFalse(pid_env_id.ancestors[1].active)

    def test_filter_and_insert(self):
        pid_env_id = PidEnvId()

        env = {}
        self.assertEquals(pid_env_id.filter_and_insert(env), PidEnvIdStatus.OK)
        self.assertFalse(pid_env_id.ancestors[0].active)

        env = {"%s%d" % (PidEnvIdEntry.prefix, 1): 2}
        self.assertEquals(pid_env_id.filter_and_insert(env), PidEnvIdStatus.OK)
        self.assertTrue(pid_env_id.ancestors[0].active)

    def test_append_direct(self):
        pid_env_id = PidEnvId()
        pid_env_id.append_direct(1, 2)
        self.assertTrue(
            pid_env_id.ancestors[0].envid.startswith(PidEnvIdEntry.prefix))

    def test_format(self):
        envid = PidEnvId.format_to_envid(1, 2)
        parent_pid, child_pid = PidEnvId.format_from_envid(envid)
        self.assertEquals((parent_pid, child_pid), (1, 2))
