#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.process.timeslice import Timeslice


class TestTimeslice(TestCase):

    def test_last_duration(self):
        timeslice = Timeslice()
        self.assertEquals(timeslice.get_last_duration(), 0.0)

        timeslice.set_finish_time_now()
        self.assertNotEquals(timeslice.get_last_duration(), 0.0)

        timeslice.reset()
        self.assertEquals(timeslice.get_last_duration(), 0.0)

    def test_timeslice(self):
        timeslice = Timeslice()
        self.assertEquals(timeslice.get_timeslice(), 0.0)
        self.assertEquals(timeslice.get_next_start_time(), 0.0)

        timeslice.set_timeslice(1.0)
        self.assertEquals(timeslice.get_timeslice(), 1.0)
        self.assertNotEquals(timeslice.get_next_start_time(), 0.0)

    def test_min_interval(self):
        timeslice = Timeslice()
        self.assertEquals(timeslice.get_min_interval(), 0.0)
        self.assertEquals(timeslice.get_next_start_time(), 0.0)

        timeslice.set_min_interval(1.0)
        self.assertEquals(timeslice.get_min_interval(), 1.0)
        self.assertNotEquals(timeslice.get_next_start_time(), 0.0)

    def test_max_interval(self):
        timeslice = Timeslice()
        self.assertEquals(timeslice.get_max_interval(), 0.0)
        self.assertEquals(timeslice.get_next_start_time(), 0.0)

        timeslice.set_max_interval(1.0)
        self.assertEquals(timeslice.get_max_interval(), 1.0)
        self.assertNotEquals(timeslice.get_next_start_time(), 0.0)

    def test_default_interval(self):
        timeslice = Timeslice()
        self.assertEquals(timeslice.get_default_interval(), 0.0)
        self.assertEquals(timeslice.get_next_start_time(), 0.0)

        timeslice.set_default_interval(1.0)
        self.assertEquals(timeslice.get_default_interval(), 1.0)
        self.assertNotEquals(timeslice.get_next_start_time(), 0.0)

    def test_initial_interval(self):
        timeslice = Timeslice()
        self.assertEquals(timeslice.get_initial_interval(), -1.0)
        self.assertEquals(timeslice.get_next_start_time(), 0.0)

        timeslice.set_initial_interval(1.0)
        self.assertEquals(timeslice.get_initial_interval(), 1.0)
        self.assertNotEquals(timeslice.get_next_start_time(), 0.0)

    def test_start_time(self):
        timeslice = Timeslice()
        self.assertEquals(timeslice.get_start_time(), 0.0)

        timeslice.set_start_time_now()
        self.assertNotEquals(timeslice.get_start_time(), 0.0)
