#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os
from time import time

from unittest import TestCase

from checkbox.process.monitor import Monitor
from checkbox.process.tracker import (
    Tracker,
    TrackerTag,
    TrackerTable,
    )


class TestTag(TrackerTag):

    __slots__ = (
        "_info",
        )

    def __init__(self, info):
        self._info = info

    def test(self, info):
        return info == self._info


class TestTracker(TestCase):

    def test_init(self):
        pid = os.getpid()
        monitor = Monitor(pid, time(), 0)
        Tracker(monitor)


class TestTrackerTable(TestCase):

    def test_addMapping(self):
        table = TrackerTable()
        self.assertEquals(table.findFamily(1), None)

        table.addMapping("foo", TestTag(1))
        self.assertEquals(table.findFamily(1), "foo")

    def test_removeMapping(self):
        table = TrackerTable()
        table.addMapping("foo", TestTag(1))
        self.assertEquals(table.findFamily(1), "foo")

        table.removeMapping("foo")
        self.assertEquals(table.findFamily(1), None)
