#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from unittest import TestCase

from checkbox.process.tree import Tree


class TestTree(TestCase):

    def test_init(self):
        tree = Tree(1)
        self.assertEquals(tree.getData(), 1)
        self.assertEquals(tree.getParent(), None)
        self.assertEquals(tree.getChild(), None)
        self.assertEquals(tree.getSibling(), None)

    def test_getChildren(self):
        root = Tree(1)
        children = list(root.getChildren())
        self.assertEquals(len(children), 0)

        root.addChild(2)
        children = list(root.getChildren())
        self.assertEquals(len(children), 1)

        root.addChild(3)
        children = list(root.getChildren())
        self.assertEquals(len(children), 2)

    def test_addChild(self):
        root = Tree(1)
        child1 = root.addChild(2)
        self.assertEquals(root.getChild(), child1)
        self.assertEquals(child1.getData(), 2)
        self.assertEquals(child1.getParent(), root)
        self.assertEquals(child1.getChild(), None)
        self.assertEquals(child1.getSibling(), None)

        child2 = root.addChild(3)
        self.assertEquals(root.getChild(), child2)
        self.assertEquals(child2.getData(), 3)
        self.assertEquals(child2.getParent(), root)
        self.assertEquals(child2.getChild(), None)
        self.assertEquals(child2.getSibling(), child1)

        child3 = child2.addChild(4)
        self.assertEquals(child2.getChild(), child3)
        self.assertEquals(child3.getData(), 4)
        self.assertEquals(child3.getParent(), child2)
        self.assertEquals(child3.getChild(), None)
        self.assertEquals(child3.getSibling(), None)

    def test_remove(self):
        root = Tree(1)
        child1 = root.addChild(2)
        self.assertEquals(root.getChild(), child1)
        child1.remove()
        self.assertEquals(root.getChild(), None)

        child1 = root.addChild(2)
        child2 = root.addChild(3)
        self.assertEquals(root.getChild(), child2)
        child2.remove()
        self.assertEquals(root.getChild(), child1)
        child1.remove()
        self.assertEquals(root.getChild(), None)

        child1 = root.addChild(2)
        child2 = child1.addChild(3)
        self.assertEquals(root.getChild(), child1)
        self.assertEquals(child1.getChild(), child2)
        child1.remove()
        self.assertEquals(root.getChild(), child2)
        child2.remove()
        self.assertEquals(root.getChild(), None)
