#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "Application",
    "ApplicationError",
    ]

import sys
import logging

from optparse import (
    OptionParser,
    OptionValueError,
    )


class ApplicationError(Exception):
    """Error raised when application fails."""

    status = 1


class Application:

    # Application defaults
    usage = "Usage: %prog [OPTIONS]"

    # Logging defaults
    default_log_level = "info"

    def __init__(self, name=None):
        """Construct new Application.

        :param name: A short name for the application.
        """
        if name is None:
            self._name = self.__class__.__name__.lower()
        else:
            self._name = name

    @property
    def name(self):
        """Enable subclasses to override with command-line arguments."""
        return unicode(self._name)

    def addOptions(self, parser):
        """Add options and option groups to the parser."""

    def parseOptions(self, options, args):
        """Parse the options as returned by the parser."""

    def process(self):
        """Process the application here. Must be defined."""
        raise NotImplementedError()

    def preProcess(self):
        """
        This should set up any state necessary before loading and
        processing the Application.
        """

    def postProcess(self, error):
        """This will be called after the processing has finished."""

    def run(self, args=None):
        """Run the application."""
        parser = OptionParser(usage=self.usage)
        self.addOptions(parser)
        options, args = parser.parse_args(args)
        try:
            self.parseOptions(options, args)
        except OptionValueError, error:
            parser.error(error)

        error = None
        self.preProcess()
        try:
            self.process()
        except ApplicationError, error:
            logging.error(str(error))
            sys.exit(error.status)
        except Exception:
            logging.exception("Unhandled exception")
            sys.exit(1)
        finally:
            self.postProcess(error)
