#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "run",
    ]

import re
import sys
import posixpath

from optparse import OptionGroup
from StringIO import StringIO

from checkbox.lib.path import path_expand_recursive
from checkbox.lib.template import Template

from checkbox.message.list import List
from checkbox.message.literal import Literal
from checkbox.message.parser import Parser
from checkbox.message.record import Record
from checkbox.message.unparser import UnparserStream

from checkbox.scripts.application import (
    Application,
    ApplicationError,
    )


CONVERT_RE = re.compile(r"(\b(or|and|float)\b|')")
CONVERT_DICT = {'or': '||', 'and': '&&', 'float': 'real', "'": "\""}
CONVERT_REPL = lambda m: CONVERT_DICT[m.group(0)]


class ConvertApplication(Application):

    # Application defaults
    usage = "Usage: %prog [OPTIONS] [FILE...]"

    def addOptions(self, parser):
        """See Application."""
        super(ConvertApplication, self).addOptions(parser)

        group = OptionGroup(parser, "Convert options")
        group.add_option("-a", "--attribute",
            action="append",
            type="string",
            default=[],
            help="Set additional attributes by name and value.")
        group.add_option("-b", "--blacklist",
            action="append",
            type="string",
            default=[],
            help="Blacklist of elements by name and value.")
        group.add_option("-w", "--whitelist",
            action="append",
            type="string",
            default=[],
            help="Whitelist of elements by name and value.")

        parser.add_option_group(group)

    def parseOptions(self, options, args):
        """See Application."""
        super(ConvertApplication, self).parseOptions(options, args)

        if args:
            self.parse_func = self.parsePaths
        else:
            self.parse_func = self.parseFile
            args = sys.stdin

        self.args = [
            args, options.attribute, options.whitelist, options.blacklist]

    def process(self):
        """See Application."""
        try:
            self.parse_func(*self.args)
        except Exception, error:
            raise ApplicationError(error.args[0])

    def parsePaths(self, paths, *args, **kwargs):
        for path in paths:
            self.parsePath(path, *args, **kwargs)

    def parsePath(self, path, *args, **kwargs):
        for filename in path_expand_recursive(path):
            name = posixpath.basename(filename)
            if name.startswith(".") or name.endswith("~"):
                continue

            file = open(filename, "r")
            self.parseFile(file, *args, **kwargs)

    def parseFile(self, file, *args, **kwargs):
        records = []
        template = Template()
        for section in template.loadFile(file):
            for key, value in section.iteritems():
                if key == "requires":
                    section[key] = self.convertFunction(value)
                elif key == "requires_extended":
                    section[key] = List([self.convertFunction(v)
                        for v in value.split("\n")])
                else:
                    section[key] = Literal(value)
            record = Record(section)
            records.append(record)

        for record in records:
            print UnparserStream.unparseRecord(record)

    def convertFunction(self, value):
        parser = Parser()
        value = CONVERT_RE.sub(CONVERT_REPL, value)
        file = StringIO(value)
        return parser.parseExpression(file, True)


def run():
    application = ConvertApplication()
    application.run()
