#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "run",
    ]

import sys

from StringIO import StringIO

from checkbox.message.locals import (
    EvalState,
    Parser,
    ParserError,
    Record,
    Unparser,
    )

from checkbox.scripts.application import Application


class MessageApplication(Application):

    # Application defaults
    usage = "Usage: %prog [FILE...]"

    def parseOptions(self, options, args):
        """See Application."""
        super(MessageApplication, self).parseOptions(options, args)

        if not args:
            args = ["-"]

        self.paths = args
        self.state = EvalState()

    def process(self):
        """See Application."""
        for path in self.paths:
            self.parsePath(path)

    def parsePath(self, path):
        if path == "-":
            file = sys.stdin
        else:
            file = open(path, "r")

        self.parseFile(file)

    def parseFile(self, file):
        buffer = []
        sys.stdout.write(">>> ")
        while True:
            line = file.readline()
            if not line:
                break

            line = line.strip()
            if not line:
                print self.parseString("".join(buffer))
                buffer = []
                sys.stdout.write(">>> ")
            else:
                buffer.append(line)
                sys.stdout.write("... ")

    def parseString(self, string):
        parser = Parser()
        try:
            expression = parser.parseExpression(StringIO(string), False)
        except ParserError, error:
            return str(error)

        if not expression:
            return "Syntax error"
        elif isinstance(expression, Record):
            self.state.setScope(expression)

        (value, expression) = expression.flatten(self.state)
        if expression is None:
            return value
        else:
            return Unparser.unparseExpression(expression)


def run():
    application = MessageApplication()
    application.run()
