#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

import os

from optparse import OptionValueError

from checkbox.tests.cases import TestCase
from checkbox.tests.env import EnvMixin
from checkbox.tests.temp import TempMixin
from checkbox.tests.unique import UniqueMixin

from checkbox.scripts.mixins.config import read_config


class TestParseConfigFiles(EnvMixin, TempMixin, UniqueMixin, TestCase):

    def test_no_file(self):
        path = self.makeFile()
        self.assertEquals(read_config(path), False)

    def test_empty_file(self):
        path = self.makeFile(content="")
        self.assertEquals(read_config(path), True)

    def test_invalid_file(self):
        path = self.makeFile(content="foo")
        self.assertRaises(OptionValueError, read_config, path)

    def test_set_environment(self):
        key = self.getUniqueString().upper()
        value = self.getUniqueString()
        path = self.makeFile(content="""\
[DEFAULT]
%s = %s
""" % (key, value))

        self.assertEquals(read_config(path), True)
        self.assertEquals(os.environ[key], value)

    def test_no_set_environment(self):
        key = self.getUniqueString().upper()
        value1 = self.getUniqueString()
        os.environ[key] = value1

        value2 = self.getUniqueString()
        path = self.makeFile(content="""\
[DEFAULT]
%s = %s
""" % (key, value2))

        self.assertEquals(read_config(path), True)
        self.assertEquals(os.environ[key], value1)
