#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "AddressOption",
    "PortOption",
    "check_address",
    "check_port",
    ]

import re

from copy import copy
from optparse import (
    Option,
    OptionValueError,
    )


ADDRESS_PATTERN = "(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})"
ADDRESS_RE = re.compile(r"^%s$" % ADDRESS_PATTERN)

PORT_PATTERN = "(\d+)"
PORT_RE = re.compile(r"^%s$" % PORT_PATTERN)


def check_address(option, opt, value):
    match = ADDRESS_RE.match(value)
    if not match or any(int(group) >= 2 ** 8 for group in match.groups()):
        raise OptionValueError(
            "option %s: invalid address: %r, should be like '192.168.0.1'"
            % (opt, value))

    return value


def check_port(option, opt, value):
    match = PORT_RE.match(value)
    if not match or int(match.groups()[0]) >= 2 ** 16:
        raise OptionValueError(
            "option %s: invalid port: %r, should be less than '65536'"
            % (opt, value))

    return int(value)


class AddressOption(Option):
    TYPES = Option.TYPES + ("address",)
    TYPE_CHECKER = copy(Option.TYPE_CHECKER)
    TYPE_CHECKER["address"] = check_address


class PortOption(Option):
    TYPES = Option.TYPES + ("port",)
    TYPE_CHECKER = copy(Option.TYPE_CHECKER)
    TYPE_CHECKER["port"] = check_port
