#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = []

from optparse import (
    OptionParser,
    OptionValueError,
    )
from unittest import TestCase

from checkbox.scripts.option import (
    AddressOption,
    PortOption,
    check_address,
    check_port,
    )
from checkbox.scripts.tests.cases import ApplicationTestCase


class TestCheckAddress(TestCase):

    def test_valid(self):
        self.assertEquals(check_address(None, "-a", "1.2.3.4"), "1.2.3.4")

    def test_invalid_string(self):
        self.assertRaises(
            OptionValueError, check_address, None, "-a", "1234")

    def test_invalid_octet(self):
        self.assertRaises(
            OptionValueError, check_address, None, "-a", "1.2.3.256")


class TestCheckPort(TestCase):

    def test_valid(self):
        self.assertEquals(check_port(None, "-p", "0"), 0)
        self.assertEquals(check_port(None, "-p", "65535"), 65535)

    def test_invalid_string(self):
        self.assertRaises(
            OptionValueError, check_port, None, "-a", "foo")

    def test_invalid_value(self):
        self.assertRaises(
            OptionValueError, check_port, None, "-a", "65536")


class TestAddressOption(ApplicationTestCase):

    def setUp(self):
        super(TestAddressOption, self).setUp()
        self.parser = OptionParser(option_class=AddressOption)
        self.parser.add_option("--address", type="address")

    def test_valid(self):
        self.parser.parse_args(["--address=1.2.3.4"])

    def test_invalid(self):
        self.assertRaises(
            SystemExit, self.parser.parse_args, ["--address=1234"])


class TestPortOption(ApplicationTestCase):

    def setUp(self):
        super(TestPortOption, self).setUp()
        self.parser = OptionParser(option_class=PortOption)
        self.parser.add_option("--port", type="port")

    def test_valid(self):
        self.parser.parse_args(["--port=0"])

    def test_invalid(self):
        self.assertRaises(
            SystemExit, self.parser.parse_args, ["--port=foo"])
