#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import os

import doctest
import unittest


def find_tests(testpaths=()):
    """Find all test paths, or test paths contained in the provided sequence.

    @param testpaths: If provided, only tests in the given sequence will
                      be considered.  If not provided, all tests are
                      considered.
    @return: a test suite containing the requested tests.
    """
    suite = unittest.TestSuite()
    testpaths = set(testpaths)
    testdir = os.path.dirname(os.path.dirname(__file__))
    topdir = os.path.abspath(os.path.join(testdir, os.pardir))
    for root, dirnames, filenames in os.walk(testdir):
        for filename in filenames:
            filepath = os.path.join(root, filename)
            relpath = filepath[len(topdir) + 1:]

            if (filename == "__init__.py" or
                filename.endswith(".pyc") or
                not filename.startswith("test_")):
                # Skip non-tests.
                continue

            if testpaths:
                # Skip any tests not in testpaths.
                for testpath in testpaths:
                    if relpath.startswith(testpath):
                        break
                else:
                    continue

            if filename.endswith(".py"):
                modpath = relpath.replace(os.path.sep, ".")[:-3]
                module = __import__(modpath, None, None, [""])
                suite.addTest(
                    unittest.defaultTestLoader.loadTestsFromModule(module))

    docdir = os.path.join(topdir, "doc")
    for filename in os.listdir(docdir):
        filepath = os.path.join(docdir, filename)
        relpath = filepath[len(topdir) + 1:]

        if testpaths:
            # Skip any tests not in testpaths.
            for testpath in testpaths:
                if relpath.startswith(testpath):
                    break
            else:
                continue

        if filename.endswith(".txt"):
            suite.addTest(doctest.DocFileSuite(
                filepath,
                module_relative=False,
                optionflags=doctest.ELLIPSIS))

    return suite
