#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "ProcessMixin",
    ]

import os
import signal


class ProcessMixin:

    signal = signal.SIGTERM

    def setUp(self):
        super(ProcessMixin, self).setUp()

        self._pids = []
        self._action = signal.getsignal(self.signal)

    def tearDown(self):
        super(ProcessMixin, self).tearDown()

        for pid in list(self._pids):
            self.killChild(pid)
        signal.signal(self.signal, self._action)

    def forkChild(self):
        pid = os.fork()
        if pid:
            self._pids.append(pid)

        return pid

    def killChild(self, pid):
        self._pids.remove(pid)

        os.kill(pid, self.signal)
        return os.waitpid(pid, 0)[1]

    def setSignal(self, action):
        signal.signal(self.signal, action)
