#
# Copyright (c) 2012 Canonical
#
# This file is part of Checkbox.
#
# Storm is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# Storm is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__metaclass__ = type

__all__ = [
    "TempMixin",
    ]

import os

from shutil import rmtree
from tempfile import (
    mkdtemp,
    mkstemp,
    )


class TempMixin:

    def setUp(self):
        super(TempMixin, self).setUp()

        self._tempfiles = []

    def tearDown(self):
        super(TempMixin, self).tearDown()

        for path in self._tempfiles:
            if os.path.isfile(path):
                os.unlink(path)
            elif os.path.isdir(path):
                rmtree(path)

    def makeFile(
        self, prefix="tmp", suffix="", basename=None,
        dirname=None, path=None, content=None):
        """Create a temporary file and return the path to it.

        The file is removed after the test runs.
        """
        if path is not None:
            self._tempfiles.append(path)
        elif basename is not None:
            if dirname is None:
                dirname = mkdtemp()
                self._tempfiles.append(dirname)
            path = os.path.join(dirname, basename)
        else:
            fd, path = mkstemp(suffix, prefix, dirname)
            self._tempfiles.append(path)
            os.close(fd)
            if content is None:
                os.unlink(path)

        if content is not None:
            file = open(path, "w")
            try:
                file.write(content)
            finally:
                file.close()

        return path

    def makeDir(self, prefix="tmp", suffix="", dirname=None, path=None):
        """Create a temporary directory and return the path to it.

        The directory is removed after the test runs.
        """
        if path is not None:
            os.makedirs(path)
        else:
            path = mkdtemp(suffix, prefix, dirname)

        self._tempfiles.append(path)
        return path
